VecBuilder

Struct VecBuilder 

pub struct VecBuilder { /* private fields */ }
Used to build a new vector.

This struct is created by the builder method on Vec.

Implementations

Source

impl VecBuilder

pub const fn capacity(&mut self, capacity: usize) -> &mut Self

The built Vec will have the minimum capacity required for capacity elements.

The capacity can be greater due to the alignment to the page size.

pub const fn growth_strategy( &mut self, growth_strategy: GrowthStrategy, ) -> &mut Self

The built Vec will have the given growth_strategy.

Panics

Panics if growth_strategy isn’t valid per the documentation of GrowthStrategy.

pub fn build<T>(&self) -> Vec<T>

Builds the Vec.

Panics
  • Panics if the max_capacity would exceed isize::MAX bytes.
  • Panics if the capacity is greater than the max_capacity.
  • Panics if reserving the allocation returns an error.
pub fn try_build<T>(&self) -> Result<Vec<T>, TryReserveError>

Tries to build the Vec, returning an error when allocation fails.

Errors
  • Returns an error if the max_capacity would exceed isize::MAX bytes.
  • Returns an error if the capacity is greater than the max_capacity.
  • Returns an error if reserving the allocation returns an error.