pub fn page_size() -> usizeExpand description
Returns the page size of the system.
The value is cached globally and very fast to retrieve.
You can align regions to the page size using align_up and align_down.
pub fn page_size() -> usizeReturns the page size of the system.
The value is cached globally and very fast to retrieve.
You can align regions to the page size using align_up and align_down.