pub struct VecBuilder { /* private fields */ }Implementations
Sourceimpl VecBuilder
impl VecBuilder
Sourcepub const fn capacity(&mut self, capacity: usize) -> &mut Self
pub const fn capacity(&mut self, capacity: usize) -> &mut Self
The built Vec will have the minimum capacity required for capacity elements.
The capacity can be greater due to the alignment to the page size.
Sourcepub const fn growth_strategy(
&mut self,
growth_strategy: GrowthStrategy,
) -> &mut Self
pub const fn growth_strategy( &mut self, growth_strategy: GrowthStrategy, ) -> &mut Self
The built Vec will have the given growth_strategy.
Panics
Panics if growth_strategy isn’t valid per the documentation of GrowthStrategy.
Sourcepub fn try_build<T>(&self) -> Result<Vec<T>, TryReserveError>
pub fn try_build<T>(&self) -> Result<Vec<T>, TryReserveError>