#[non_exhaustive]pub enum GrowthStrategy {
Exponential {
numerator: usize,
denominator: usize,
},
Linear {
elements: usize,
},
}Expand description
The strategy to employ when growing a vector.
Note that arithmetic overflow is not a concern for any of the strategies: if the final result
of the calculation (in infinite precision) exceeds max_capacity, max_capacity is used as
the new capacity.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Exponential
The current capacity is multiplied by numerator, then divided by denominator.
If the new capacity results in the last committed page having spare room for more elements, those elements are added to the new capacity.
Fields
Linear
elements is added to the current capacity.
If the new capacity results in the last committed page having spare room for more elements, those elements are added to the new capacity.
Implementations